package neustore.base;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * Interface for all database objects.
 * 
 * @version $Id: DBObject.java 62 2009-06-19 08:32:04Z sveinbra $
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public interface DBObject<T> {
    /**
     * Creates a new copy of the object.
     * @return  the new copy
     */
    public Object clone();
    /* 
     * Returns the number of bytes the object occupy.
     * @return  number of bytes
     */
    public int size();
    /**
     * Returns the maximum number of bytes this type of object may occupy.
     * It is used to support variable-length data.
     * @return  maximum number of bytes
     */
    public int maxSize();
    /**
     * Reads the object from an input stream.
     * @param in   input stream
     * @throws IOException
     */
    public void read(DataInputStream in) throws IOException ;
    /**
     * Writes the object to an output stream.
     * @param out   output stream
     * @throws IOException
     */
    public void write(DataOutputStream out) throws IOException ;

    /**
     * @return the value of the given data object.
     */
    public T value(); 
}
