package neustore.base;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * Float data type.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class FloatData implements Data<Float>, Key<Float> {
    private float value;

    public FloatData(float value) {
        this.value = value;
    }

    public Object clone() {
        return new FloatData(value);
    }

    public int size() {
        return 4;
    }

    public int maxSize() {
        return size();
    }

    public void read(DataInputStream in) throws IOException {
        value = in.readFloat();
    }

    public void write(DataOutputStream out) throws IOException {
        out.writeFloat(value);
    }

    public int compareTo(Key<?> key2) {
        return value().compareTo((Float)key2.value());
    }

    public Float value() {
        return value;
    }
    
    @Override
    public int hashCode() {
        return value().hashCode();
    }
    
    @Override
    public boolean equals(Object obj) {
        return value().equals(((IntKey)obj).value());
    }

    @Override
    public String toString() {
        return getClass().getSimpleName() + "[value=" + value() + "]";
    }
}
