package neustore.base;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.HashMap;
import java.util.Map;

/**
 * The Main Memory buffer.
 * 
 * @author Svein Erik Bratsberg &lt;sveinbra@idi.ntnu.no&gt;
 */
public class MMBuffer extends DBBuffer {
	protected Map<Integer, DBBufferStoredElement> buffer;
	
	public MMBuffer(int bufferSize, int pageSize) {
		super(bufferSize, pageSize);
		buffer = new HashMap<Integer, DBBufferStoredElement>();
	}

	/**
	 * Finds a page in buffer.
	 * 
	 * @param pageID    the page ID
	 * @return an instance of DBBufferStoredElement if found; <i>null</i>otherwise.
	 */
	protected DBBufferStoredElement find(RandomAccessFile file, int pageID) {
	    return buffer.get(pageID);
	}
	
	/**
	 * Adds a page to the buffer.
	 * Note: do not increase bufferReadIO.
	 * The element should not exist in the buffer already.
	 * 
	 * @param stored   the stored element
	 */
	protected void add(DBBufferStoredElement stored ) throws IOException{
		buffer.put( stored.pageID, stored );
	}
	
	/**
	 * Flushes the buffer pages of a given file. 
	 * Makes the buffer empty, while saving modified pages to disk.
	 * 
	 * @throws IOException
	 */
	protected void flush (RandomAccessFile file) throws IOException {
	}
}
