package neustore.heapfile;

import java.io.IOException;
import java.util.ArrayDeque;
import java.util.Deque;

import neustore.base.DBRuntimeException;
import neustore.base.Data;
import neustore.base.Key;
import ntnustore.dlaesa.Cursor;

/**
 * TODO: Description of type.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class HeapFileCursor<K extends Key<?>, D extends Data<?>> implements Cursor<K, D> {
    private HeapFile<K, D> heapFile;
    private Deque<Integer> pageIds;
    private HeapFilePage<K, D> currentPage;
    private HeapFileRecord<K, D> currentRecord;
    private int nextRecord;

    public HeapFileCursor(HeapFile<K, D> heapFile) {
        this.heapFile = heapFile;
        this.pageIds = new ArrayDeque<Integer>();
        pageIds.push(heapFile.firstFullPageID);
        pageIds.push(heapFile.firstNonFullPageID);
    }

    public boolean next() {
        // if no current page, or the current page is done, read next
        while (currentPage == null || (nextRecord >= currentPage.records.size())) {
            if (pageIds.isEmpty()) {
                return false;
            } else {
                int pageId = pageIds.pop();
                if (pageId < 0) {
                    continue;
                }
                
                try {
                    currentPage = heapFile.myReadPage(pageId);
                } catch (IOException e) {
                    throw new DBRuntimeException(e);
                }

                pageIds.push(currentPage.next);
                nextRecord = 0;
            }
        }

        currentRecord = currentPage.records.get(nextRecord);
        nextRecord++;
        return true;
    }

    public K getKey() {
        return currentRecord.key;
    }

    public D getData() {
        return currentRecord.data;
    }
}
