package ntnustore.btree;

import java.io.File;
import java.io.IOException;
import java.util.Random;
import java.util.Map;

import neustore.base.DBBuffer;
import neustore.base.DBIndex;
import neustore.base.LRUBuffer;
import neustore.base.IntKey;
import neustore.base.StringData;
import neustore.base.DoubleKey;
import neustore.base.IntData;

import ntnustore.relalg.HashJoin;

public class TestBTree {
    final static int maxStringLength = 100;

    BTree testCreate (DBBuffer buffer, String filename ) throws IOException {
	File file = new File(filename);
	file.delete();
	DoubleKey sampleKey = new DoubleKey(0.0);
	IntData sampleData = new IntData(0); 
	BTree bt = new BTree (buffer, filename, DBIndex.CREATE, sampleKey, sampleData);
	return bt;
    }
	

    public static void main(String[]args) throws IOException {
	System.out.println( "***** Creating databases *****" );		
	String filename = "TESTBTREE";
	LRUBuffer buffer = new LRUBuffer( 10000, 8192);
	TestBTree test = new TestBTree();		
	BTree bt = test.testCreate(buffer,filename);
	
	int nRecs = Integer.valueOf(args[0]).intValue();
	System.out.println( "***** Inserting keys *****");
	long t0 = System.currentTimeMillis();

        Random key = new Random();
        //nRecs = 100;
	for (int i=0;i<nRecs;i++) {
	    try {
		bt.insert( new DoubleKey(key.nextFloat()), new IntData(i));
	    } catch (IOException e) {}
	}
        
        BTree bt2 = test.testCreate(buffer, "TESTBTREE2");
	for (int i=0;i<nRecs;i++) {
	    try {
		bt2.insert( new DoubleKey(key.nextFloat()), new IntData(i));
	    } catch (IOException e) {}
	}
/*
        BTree bt3 = test.testCreate(buffer, "TESTBTREE3");
	for (int i=0;i<nRecs;i++) {
	    try {
		bt3.insert( new DoubleKey(key.nextFloat()), new IntData(i));
	    } catch (IOException e) {}
	}
 */
	long t1 = System.currentTimeMillis();
	System.out.println("Insert time="+(t1-t0));

	// bt.debugOn = true;

	System.out.println( "***** Joining keys *****" );

	t0 = System.currentTimeMillis();
        
        // NestedLoopJoin myJoin = new NestedLoopJoin (bt.scanFrom(new DoubleKey(0.0)),bt2.scanFrom(new DoubleKey(0.0)));
        // myJoin.execute();
        HashJoin myJoin = new HashJoin (1777);
        myJoin.add(bt.scanFromTo(new DoubleKey(0.0),new DoubleKey(1.0)), nRecs);
        myJoin.add(bt2.scanFromTo(new DoubleKey(0.0),new DoubleKey(1.0)), nRecs);
        //myJoin.add(bt3.scanFrom(new DoubleKey(0.0)), nRecs);
        myJoin.partition();
        t1 = System.currentTimeMillis();
        System.out.println("Join partition time="+(t1-t0));
        myJoin.probe();
        
        t0 = System.currentTimeMillis();
	System.out.println("Join probe time="+(t0-t1));	     
	// StringData sd;

//	for (int i=0; i<1000000;i++)
//	    sd= (StringData)bt.search(new IntKey(key.nextInt(nRecs)));
/*
        for (int i=0; i<10;i++) {
            BTreeCursor myCurs = bt.scanFrom(new DoubleKey(key.nextDouble()));
            Map.Entry myRec;
            int j = 0;
            while ((myRec = myCurs.next())!=null && j<100) {
                System.out.println("Key="+((DoubleKey)myRec.getKey()).key+" Value="+((IntData)myRec.getValue()).dataValue);
                j++;
            }
        }
	t1 = System.currentTimeMillis();
	System.out.println("Search time="+(t1-t0));
*/
	/*
	System.out.println( "***** Deleting key=60 from bp *****" );
	bt.delete( new IntKey(60) );

	System.out.println( "***** Searching keys 60 *****" );
	sd = (StringData)bt.search( new IntKey(60) );
	if ( sd != null ) {
	    System.out.println("Strange, key=60 was deleted but still found." );
	}
	else {
	    System.out.println("Good, key=60 was deleted and thus not found.");
	}
         */
	bt.close();
        bt2.close();
    }
}

