package ntnustore.distance;

/**
 * Interface for all classes performing distance calculations between two
 * objects of type T.
 * 
 * @version $Id: Distance.java 48 2008-11-02 10:33:17Z erikbagg $
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public interface Distance<T> {

    /**
     * Initialize the distance calculations with the given base object.
     * 
     * @param o1 object to calculate distances from.
     */
    public abstract void initialize(T o1);

    /**
     * Returns the distance from the initialized object.
     * 
     * @param o2 object to calculate distance to.
     * @return distance between base object and o2.
     */
    public abstract float distance(T o2);

    /**
     * @return the total number of distance calculations performed.
     */
    public abstract int getCalculations();

}