package ntnustore.distance;

import neustore.base.StringData;
import ntnustore.distance.Distance;

public class LevenshteinWordDistance implements Distance<StringData> {
    private int[] char1;
    private int calculations;

    private final static int COST_INSERT = 1;
    private final static int COST_DELETE = 1;
    private final static int COST_MODIFY = 1;

    public LevenshteinWordDistance() {
        super();
    }

    public LevenshteinWordDistance(StringData o1) {
        this();
        initialize(o1);
    }

    public void initialize(StringData o1) {
    	char1 = wordHashes(o1.value());
    }
    
    private int[] wordHashes(String string) {
    	String s[] = string.split("\\s+");
    	int hash[] = new int[s.length];
    	for (int i = 0; i < s.length; i++) {
			hash[i] = s[i].hashCode();
		}
    	return hash;
    }

    public float distance(StringData o2) {
        calculations++;
        // prepare operands
        int[] char1 = this.char1;
        int[] char2 = wordHashes(o2.value());
        int len1 = char1.length;
        int len2 = char2.length;

        // create and initialize result array
        int d1[] = new int[len1 + 1];
        for (int i = 0; i <= len1; i++) {
            d1[i] = i;
        }

        // iterate over each character in o2
        for (int i = 1; i <= len2; i++) {
            int n2 = i; // n is used as a faster d2[j-1]
            int n1 = d1[0]; // n1 is used as a faster d1[j-1]
            int c2 = char2[i - 1]; // c2 is used as a faster char2[i-1]
            d1[0] = i;

            for (int j = 1; j <= len1; j++) {
                // find what is cheapest for this letter, inserting, deleting or
                // modifying
                n2 = Math.min(Math.min(d1[j] + COST_INSERT, n2 + COST_DELETE), n1
                        + (c2 == char1[j - 1] ? 0 : COST_MODIFY));

                // n1 (d1[j-1]) is used in the next iteration
                n1 = d1[j];
                d1[j] = n2;
            }
        }
        return d1[len1];
    }

    public int getCalculations() {
        return this.calculations;
    }
}
