package ntnustore.dlaesa;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import neustore.base.DBRuntimeException;

/**
 * Buffered, Runnable cursor for use in execution parallelization.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class BufferedCursor<K, D> implements Cursor<K, D>, Runnable {
    private Cursor<K, D> cursor;
    private BlockingQueue<Pair<K, D>> buffer;
    private Pair<K, D> current;
    private Pair<K, D> DONE = new Pair<K, D>(null, null);

    public BufferedCursor(Cursor<K, D> cursor, int capacity) {
        super();
        this.cursor = cursor;
        this.buffer = new LinkedBlockingQueue<Pair<K, D>>(capacity);
    }

    public boolean next() {
        try {
            current = buffer.take();
        } catch (InterruptedException e) {
            throw new DBRuntimeException(e);
        }
        return !DONE.equals(current);
    }

    public K getKey() {
        return current.key;
    }

    public D getData() {
        return current.value;
    }

    public void run() {
        while (cursor.next()) {
            Pair<K, D> p = new Pair<K, D>(cursor.getKey(), cursor.getData());
            try {
                buffer.put(p);
            } catch (InterruptedException e) {
                throw new DBRuntimeException(e);
            }
        }
        buffer.add(DONE);
    }
}
