package ntnustore.dlaesa;

import java.io.File;
import java.io.IOException;

import neustore.base.DBBuffer;
import neustore.base.DBRuntimeException;
import neustore.base.Data;
import neustore.base.FloatData;
import neustore.base.IntData;
import neustore.base.IntKey;
import neustore.heapfile.HeapFile;
import ntnustore.distance.Distance;

/**
 * TODO: Description of type.
 * 
 * @version $Id: $
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class LaesaHeapFile<D extends Data<?>> extends HeapFile<IntKey, FloatData> implements LaesaIndex<D> {
    private Distance<D> distance;

    public LaesaHeapFile(DBBuffer buffer, String fileName, Distance<D> distance) throws IOException {
        super(buffer, fileName, !new File(fileName).exists(), new IntKey(0), new FloatData(0.0F));

        this.distance = distance;
    }

    public void insert(IntKey key, D data) {
        float dist = distance.distance(data);

        try {
            insert(key, new FloatData(dist));
        } catch (IOException e) {
            // TODO shouldn't throw IOException
            throw new DBRuntimeException(e);
        }
    }
    
	@Override
	public float insert2(IntKey key, D data) {
        float dist = distance.distance(data);

        try {
        	insert(key, new FloatData(dist));
        } catch (IOException e) {
            // TODO shouldn't throw IOException
            throw new DBRuntimeException(e);
        }
        return dist;
	}

    public Cursor<IntData, Float> query(D query, float range) {
        float dist = distance.distance(query);

        //Cursor<FloatData, IntData> cursor = query(new FloatData(dist - range), new FloatData(dist + range));

        //return new HeapFileLowerBoundCursor<IntData>(cursor, dist);
        return null;
    }

    /**
     * Private, static inner class used by query() for transforming the Cursor<FloatKey,
     * K> returned from RTree into a Cursor<K, Float> for use in further
     * LAESA-handling.
     * 
     * The Float returned is the greatest lower bound the pivot can calculate
     * for each key.
     */
    private static class HeapFileLowerBoundCursor<K> implements Cursor<K, Float> {
        private Cursor<FloatData, K> cursor;
        private float dist;

        public HeapFileLowerBoundCursor(Cursor<FloatData, K> cursor, float dist) {
            super();
            this.cursor = cursor;
            this.dist = dist;
        }

        public boolean next() {
            return cursor.next();
        }

        public K getKey() {
            return cursor.getData();
        }

        public Float getData() {
            FloatData data = cursor.getKey();
            return Math.abs(data.value() - dist);
        }
    }

    public float costOfQuery (D query, float range) {
        return 0.0F;
    }
    
    /**
     * TODO: Generate statistics for query cost estimation
     */
    public void generateStatistics () {
    }





}
