package ntnustore.dlaesa;

import java.io.IOException;

import neustore.base.DBIndex;
import neustore.base.IntData;
import neustore.base.IntKey;

/**
 * A generalized interface for all LAESA index implementations.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 * @param <D>
 */
public interface LaesaIndex<D> {

    /**
     * Insert a single record into this index.
     * 
     * @param key key of record
     * @param data data of record (only used for indexing, not stored in the actual index).
     */
    public abstract void insert(IntKey key, D data);
    
    public abstract float insert2(IntKey key, D data);

    /**
     * Returns all keys within the given range of query as seen from this index.
     * As well as either an exact distance or the highest lower bound that can
     * be determined by the index.
     * 
     * @param query object to search for
     * @param range how far from the query object a result object can be, as
     *        seen from the pivots in this RTree
     * @return a cursor with the key and (highest lower bound) distance data.
     */
    public abstract Cursor<IntData, Float> query(D query, float range);

    /**
     * Estimate cost of query
     * @param query
     * @param range
     * @return
     */
    public abstract float costOfQuery(D query, float range);
    /**
     * Generate statistics for this index.
     */

    public abstract void generateStatistics ();

    /**
     * @see DBIndex#close()
     */
    public void close () throws IOException;
}