/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ntnustore.dlaesa;

import java.io.File;
import java.io.IOException;
import org.w3c.dom.Document;
import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException; 

import neustore.base.*;
import ntnustore.btree.*;
import ntnustore.relalg.*;

/**
 *
 * @author Svein Erik
 */
public class ReadGeneratedData {

    public static BTree createTable (DBBuffer buffer, String filename ) throws IOException {
	File file = new File(filename);
	file.delete();
	DoubleKey sampleKey = new DoubleKey(0.0);
	IntData sampleData = new IntData(0); 
	BTree bt = new BTree (buffer, filename, DBIndex.CREATE, sampleKey, sampleData);
	return bt;
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // TODO code application logic here
     try {

            DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
            Document doc = docBuilder.parse (new File("output.xml"));

            // normalize text representation
            doc.getDocumentElement ().normalize ();
            System.out.println ("Root element of the doc is " + 
                 doc.getDocumentElement().getNodeName());
            
            LRUBuffer buffer = new LRUBuffer( 10000, 8192);
            int nDimensions = 0;
            double radius = 0.0;
            
            NodeList settings = doc.getElementsByTagName("settings");
            if (settings.getLength()>=1) {
                Element settingsObj = (Element)settings.item(0);
                NodeList dimensionList = settingsObj.getElementsByTagName("dimension");
                if (dimensionList.getLength()>=1) {
                    Element dimensionElement = (Element)dimensionList.item(0);
                    NodeList textFNList = dimensionElement.getChildNodes();
                    nDimensions = Integer.parseInt(((Node)textFNList.item(0)).getNodeValue().trim());
                    System.out.println(" dimensions=" + nDimensions);
                }
                dimensionList = settingsObj.getElementsByTagName("radius");
                if (dimensionList.getLength()>=1) {
                    Element dimensionElement = (Element)dimensionList.item(0);
                    NodeList textFNList = dimensionElement.getChildNodes();
                    radius = Double.parseDouble(((Node)textFNList.item(0)).getNodeValue().trim());
                    System.out.println(" radius=" + radius);
                }
            }
            
            BTree tables[] = new BTree[nDimensions]; 
            for (int b=0;b<nDimensions;b++) {
                tables[b] = createTable (buffer, "table"+b);
            }
            
            NodeList listOfObjects = doc.getElementsByTagName("object");
            int totalObjects = listOfObjects.getLength();
            System.out.println("Total no of objects : " + totalObjects);

            for(int o=0; o<totalObjects; o++){
                Node objectNode = listOfObjects.item(o);
                if(objectNode.getNodeType() == Node.ELEMENT_NODE){


                    Element objectElement = (Element)objectNode;
                    int id = Integer.parseInt(objectElement.getAttribute("id"));
                    //System.out.println("id="+id);
                    
                    NodeList pivotList = objectElement.getElementsByTagName("dist");
                    int nDist = pivotList.getLength();
                    for (int d=0;d<nDist;d++) {
                        Element distElement = (Element)pivotList.item(d);
                        int pivot = Integer.parseInt(distElement.getAttribute("pivot"));
                        //System.out.print("pivot="+pivot);
                        NodeList textFNList = distElement.getChildNodes();
                        double dist = Double.parseDouble(((Node)textFNList.item(0)).getNodeValue().trim());
                        //System.out.println(" dist=" + dist);
                        try {
                            tables[pivot-1].insert( new DoubleKey(dist), new IntData(id));
                        } catch (IOException e) {}
                    }
                }
            }
            
            NodeList listOfQueries = doc.getElementsByTagName("query");
            int totalQueries = listOfQueries.getLength();
            for (int q=0;q<totalQueries;q++) {
                Node queryNode = listOfQueries.item(q);
                if(queryNode.getNodeType() == Node.ELEMENT_NODE){
                    Element queryElement = (Element)queryNode;
                    int qid = Integer.parseInt(queryElement.getAttribute("id"));
                    System.out.println("query id="+qid);
                    HashJoin myJoin = new HashJoin(1777);
                    NodeList pivotList = queryElement.getElementsByTagName("dist");
                    int nDist = pivotList.getLength();
                    for (int d=0;d<nDist;d++) {
                        Element distElement = (Element)pivotList.item(d);
                        int pivot = Integer.parseInt(distElement.getAttribute("pivot"));
                        //System.out.print("pivot="+pivot);
                        NodeList textFNList = distElement.getChildNodes();
                        double dist = Double.parseDouble(((Node)textFNList.item(0)).getNodeValue().trim());
                        //System.out.println(" dist=" + dist);
                        myJoin.add(tables[d].scanFromTo(new DoubleKey(dist-radius),new DoubleKey(dist+radius)), totalObjects);
                    }
                    long t0 = System.currentTimeMillis();
                    myJoin.partition();
                    long t1 = System.currentTimeMillis();
                    System.out.println("Join partition time="+(t1-t0));
                    myJoin.probe();
        
                    t0 = System.currentTimeMillis();
                    System.out.println("Join probe time="+(t0-t1));
            
                    System.out.println("nRecs in result="+myJoin.resultSize());

                }
            }
            
            for (int b=0; b<nDimensions;b++) {
                tables[b].close();
            }


        }catch (SAXParseException err) {
        System.out.println ("** Parsing error" + ", line " 
             + err.getLineNumber () + ", uri " + err.getSystemId ());
        System.out.println(" " + err.getMessage ());

        }catch (SAXException e) {
        Exception x = e.getException ();
        ((x == null) ? e : x).printStackTrace ();

        }catch (Throwable t) {
        t.printStackTrace ();
        }
        //System.exit (0);
   }

}
