package ntnustore.dlaesa;

import java.util.Arrays;
import java.util.List;

/**
 * Union of all keys and data from the cursors given in the constructor in
 * order. Does NOT ensure unique items, equal to SQL UNION ALL.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class UnionCursor<K, D> implements Cursor<K, D> {
    private List<Cursor<K, D>> cursors;

    public UnionCursor(Cursor<K, D>... cursors) {
        this.cursors = Arrays.asList(cursors);
    }

    public boolean next() {
        while (cursors.size() > 0) {
            if (cursors.get(0).next()) {
                return true;
            } else {
                cursors.remove(0);
            }
        }
        return false;
    }

    public K getKey() {
        return cursors.get(0).getKey();
    }

    public D getData() {
        return cursors.get(0).getData();
    }
}
