package ntnustore.rtree;

import java.util.ArrayDeque;
import java.util.Deque;

import neustore.base.IntData;
import ntnustore.dlaesa.Cursor;

/**
 * TODO: Description of type.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class RTreeRangeQueryCursor implements Cursor<RTreeRegion, IntData> {
    private RTree rtree;
    private RTreeRegion query;
    private Deque<Integer> unvisitedNodes;
    
    private RTreeRecord<IntData> currentRecord;
    private Deque<RTreeRecord<IntData>> currentRecords;

    public RTreeRangeQueryCursor(RTree rtree, RTreeRegion query, Integer rootNodeId) {
        super();
        this.rtree = rtree;
        this.query = query;
        this.unvisitedNodes = new ArrayDeque<Integer>();
        this.unvisitedNodes.add(rootNodeId);
        this.currentRecords = new ArrayDeque<RTreeRecord<IntData>>();
        this.currentRecord = null;
    }

    public boolean next() {
        while (true) {
            if (!currentRecords.isEmpty()) {
                currentRecord = currentRecords.pop();
                return true;
            }
    
            if (unvisitedNodes.isEmpty()) {
                currentRecord = null;
                return false;
            }
            
            Integer pageId = unvisitedNodes.pop();
            RTreePage node = rtree.readPage(pageId);
            
            RTreeRecordList<IntData> records = node.getRecords();
            for (RTreeRecord<IntData> record : records) {
                if (RTreeRegion.intersects(query, record.mbr)) {
                    if (node.level == 0) {
                        currentRecords.push(record);
                    } else {
                        // data = pageId of child
                        unvisitedNodes.add(record.data.value());
                    }
                }
            }
        }
    }
    
    public RTreeRegion getKey() {
        return currentRecord.mbr;
    }

    public IntData getData() {
        return currentRecord.data;
    }
}
