package ntnustore.rtree;

import java.util.Arrays;

/**
 * TODO: Description of type.
 * 
 * @version $Id: RTreeRegion.java 68 2010-02-19 08:56:14Z sveinbra $
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class RTreeRegion {
    public int dimensions;
    public float low[], high[];

    public RTreeRegion(int dimensions) {
        super();
        this.dimensions = dimensions;
        this.low = new float[dimensions];
        this.high = new float[dimensions];
    }

    public RTreeRegion(float[] low, float[] high) {
        this(low.length);
        System.arraycopy(low, 0, this.low, 0, dimensions);
        System.arraycopy(high, 0, this.high, 0, dimensions);
    }

    public RTreeRegion(float[] points) {
        this(points, points);
    }
    
    public double getArea() {
        double area = 1.0;
        for (int i = 0; i < dimensions; i++) {
            area *= (high[i] - low[i]);
        }
        return area;
    }

    public double getMargin() {
        double margin = 0.0F;
        for (int i = 0; i < dimensions; i++) {
            margin += (high[i] - low[i]);
        }
        return margin;
    }

    public static RTreeRegion union(RTreeRegion... regions) {
        RTreeRegion union = new RTreeRegion(regions[0].dimensions);

        for (int d = 0; d < union.dimensions; d++) {
            // for each dimension, find the lowest and highest values
            float low = regions[0].low[d];
            float high = regions[0].high[d];

            for (int r = 1; r < regions.length; r++) {
                if (regions[r].low[d] < low) {
                    low = regions[r].low[d];
                }
                if (regions[r].high[d] > high) {
                    high = regions[r].high[d];
                }
            }

            union.low[d] = low;
            union.high[d] = high;
        }

        return union;
    }

    public static RTreeRegion intersect(RTreeRegion... r) {
        RTreeRegion intersect = new RTreeRegion(r[0].low, r[0].high);
        for (int j = 1; j < r.length; j++) {
            for (int i = 0; i < intersect.dimensions; i++) {
                intersect.low[i] = Math.max(r[j].low[i], intersect.low[i]);
                intersect.high[i] = Math.min(r[j].high[i], intersect.high[i]);
            }
        }
        return intersect;
    }
    
    public static boolean intersects(RTreeRegion r1, RTreeRegion r2) {
        for (int i = 0; i < r1.dimensions; i++) {
            if ((r2.low[i] > r1.high[i]) || (r1.low[i] > r2.high[i])) {
                return false;
            }
        }
        return true;
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("RTreeRegion[dimensions=");
        sb.append(dimensions);
        for (int i = 0; i < low.length; i++) {
            sb.append(", D");
            sb.append(i+1);
            sb.append("=");
            sb.append(low[i]);
            sb.append("-");
            sb.append(high[i]);
        }
        sb.append("]");
        return sb.toString();
    }

    public boolean contains(RTreeRegion r) {
        for (int i = 0; i < dimensions; i++) {
            if ((r.low[i] < low[i]) || (r.high[i] > high[i])) {
                return false;
            }
        }
        return true;
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof RTreeRegion)) {
            return false;
        }
        RTreeRegion r = (RTreeRegion)o;
        return Arrays.equals(low, r.low) && Arrays.equals(high, r.high); 
    }
}
