package test;

import java.util.List;
import java.util.Random;

import neustore.base.FloatArrayData;
import neustore.base.StringData;
import ntnustore.distance.ChebyshevDistance;
import ntnustore.distance.Distance;
import ntnustore.distance.L2Distance;
import ntnustore.distance.LXDistance;
import ntnustore.distance.LevenshteinDistance;
import ntnustore.distance.LevenshteinWordDistance;

public class TestDistanceCost {
	private static <T> void compareDists(String caption, int count, int subCount, List<T> data, Distance<T> distance) {
		long sumTime = 0;
		float sumDist = 0.0F;
		Random r1 = new Random();
		Random r2 = new Random();

		for (int i = 0; i < count; i++) {
			long time = System.nanoTime();
			float dist = 0.0F;

			T d1 = data.get(r1.nextInt(data.size()));
			for (int j = 0; j < subCount; j++) {
				T d2 = data.get(r2.nextInt(data.size()));

				distance.initialize(d1);
				dist += distance.distance(d2);
			}

			time = System.nanoTime() - time;

			sumTime += (time / subCount);
			sumDist += (dist / subCount);
		}
		System.out.println(caption + "\t" + data.size() + "\t" + count + "*" + subCount + "\t" + (sumTime / count) + "\t" + (sumDist / count));
	}

	public static void main(String[] args) throws Exception {
		List<FloatArrayData> nasaData = TestUtil.readFloatDataFromFile("data/nasa.data");
//		compareDists("L1 (nasa)", 1000000, 1, nasaData, new LXDistance(1F));
//		compareDists("L2 (nasa)", 1000000, 1, nasaData, new L2Distance());
//		compareDists("L5 (nasa)", 1000000, 1, nasaData, new LXDistance(5F));
//		compareDists("Lmax (nasa)", 1000000, 1, nasaData, new ChebyshevDistance());

//		List<FloatArrayData> colorsData = TestUtil.readFloatDataFromFile("data/colors.data");
//		compareDists("L1 (colors)", 1000000, 1, colorsData, new LXDistance(1F));
//		compareDists("L2 (colors)", 1000000, 1, colorsData, new L2Distance());
//		compareDists("L5 (colors)", 1000000, 1, colorsData, new LXDistance(5F));
//		compareDists("Lmax (colors)", 1000000, 1, colorsData, new ChebyshevDistance());

//		List<StringData> trecData = TestUtil.readStringDataFromFile("data/GX000_002.txt", 2048);
//		compareDists("Levenshtein word (TREC)", 1000000, 1, trecData, new LevenshteinWordDistance());
//		compareDists("Levenshtein (TREC)", 100000, 1, trecData, new LevenshteinDistance());

		compareDists("L1 (nasa)", 10000, 100, nasaData, new LXDistance(1F));
		compareDists("L2 (nasa)", 10000, 100, nasaData, new L2Distance());
		compareDists("L5 (nasa)", 10000, 100, nasaData, new LXDistance(5F));
		compareDists("Lmax (nasa)", 10000, 100, nasaData, new ChebyshevDistance());

//		compareDists("L1 (colors)", 10000, 100, colorsData, new LXDistance(1F));
//		compareDists("L2 (colors)", 10000, 100, colorsData, new L2Distance());
//		compareDists("L5 (colors)", 10000, 100, colorsData, new LXDistance(5F));
//		compareDists("Lmax (colors)", 10000, 100, colorsData, new ChebyshevDistance());

//		compareDists("Levenshtein word (TREC)", 10000, 100, trecData, new LevenshteinWordDistance());
//		compareDists("Levenshtein (TREC)", 1000, 100, trecData, new LevenshteinDistance());
	}
}
