package test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import neustore.base.FloatArrayData;
import neustore.base.IntData;
import neustore.base.IntKey;
import ntnustore.distance.Distance;
import ntnustore.distance.L2Distance;
import ntnustore.dlaesa.Cursor;
import ntnustore.dlaesa.DistanceCursor;
import ntnustore.dlaesa.KnnCursor;

/**
 * TODO: Description of type.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class TestInsertionCost extends AbstractTest<FloatArrayData> {
	private static final String DATA_FILE = "data/nasa";

        //private static final String DATA_FILE = "data/unicoords";
	private void testInsertion(List<FloatArrayData> fileData, PivotCount pivotCount, int sampleSize, List<FloatArrayData> queryData, float queryRange, int queryCount)
			throws Exception {

		int sumInsertCost = 0;
		long sumInsertTime = 0;
		int sumQueryCost = 0;
		long sumQueryTime = 0;

		int objectIndex = 0;
		while (objectIndex < fileData.size()) {
			// insert
			long startTime = System.nanoTime();
			int startCost = getDistanceCost();
//			int diskWriteIO = buffer.getIOs()[1];
			
			sampleSize = Math.min(sampleSize, fileData.size() - objectIndex);
			while (pivotCount.getPivotCount(objectIndex + sampleSize) > getPivotCount()) {
				addPivot();
			}
			for (int i = 0; i < sampleSize; i++) {
				insert(fileData.get(objectIndex + i));
			}
			objectIndex += sampleSize;
			buffer.force(null);

			print(objectIndex, "\t");
			
			int cost = getDistanceCost() - startCost;
			sumInsertCost += cost;
			print(cost, "\t");
			print(sumInsertCost, "\t");

			long time = (System.nanoTime() - startTime) / 1000000;
			sumInsertTime += time;
			print(time, "\t");
			print(sumInsertTime, "\t");
			
//			diskWriteIO = buffer.getIOs()[1] - diskWriteIO;
//			print(diskWriteIO, "\t");
			
			// query
                    if ((queryData != null)) { //&& (objectIndex > 30000)) { //TODO
                        startTime = System.nanoTime();
                        startCost = getDistanceCost();
                        int count = 0;


                        for (FloatArrayData query : queryData) {
                            Cursor<IntData, Float> cursor = indexList.query(query, queryRange);
		
                            // filter inexact matches with KNN
                            Distance<FloatArrayData> distance = new L2Distance(query);
                            if (queryCount > 0) {
                                cursor = new KnnCursor<IntData, FloatArrayData>(cursor, dataFile, distance, queryRange, queryCount);
                            } else if (queryCount == 0) {
                                cursor = new DistanceCursor<IntData, FloatArrayData>(cursor, dataFile, distance, queryRange);
                            }
		
                            // count results
                            while (cursor.next()) {
                                count++;
                            }
                            startCost -= distance.getCalculations();

                            //Cursor<IntData, Float> ccursor = indexList.query(query, queryRange);
                            //while (ccursor.next()) ccount++;

                        }

                        print(count, "\t");

                        cost = getDistanceCost() - startCost;
                        sumQueryCost += cost;
                        print(cost, "\t");
                        print(sumQueryCost, "\t");

                        time = (System.nanoTime() - startTime) / 1000000;
                        sumQueryTime += time;
                        print(time, "\t");
                        print(sumQueryTime, "\t");

                        // scan the data file as a comparison
                        startTime = System.nanoTime();
                        int ccount = 0;
                        for (FloatArrayData query : queryData) {
                            Distance<FloatArrayData> distance = new L2Distance(query);
                            Cursor<IntKey, FloatArrayData> dcursor = dataFile.query(new IntKey(0), new IntKey(objectIndex));
                            while (dcursor.next()) {
                                float currentDistance = distance.distance(dcursor.getData());
                                if (currentDistance<=queryRange) ccount++;
                            }
                        }
                        time = (System.nanoTime() - startTime) / 1000000;
                        print(ccount, "\t");
                        print(time, "\t");
                    }
			
                    print("\n");
                    //System.out.println(objectIndex);
		}
	}

	private List<Distance<FloatArrayData>> readPivotList() throws IOException {
		List<Distance<FloatArrayData>> result = new ArrayList<Distance<FloatArrayData>>();

		List<FloatArrayData> pivots = TestUtil.readFloatDataFromFile(DATA_FILE + ".pivot");
		for (FloatArrayData pivot : pivots) {
			Distance<FloatArrayData> distance = new L2Distance();
			distance.initialize(pivot);
			result.add(distance);
		}

		return result;
	}
	
	public static void main(String[] args) throws Exception {
		int indexType = 4;
		int pageSize = 8192;
		int bufferSize = 10000; //TODO
		int sampleSize = 2000;
		int queryCount = 0;
		float queryRange = 0.2F;
		
		TestUtil.deleteFiles();
		TestInsertionCost t;
		List<FloatArrayData> data = TestUtil.readFloatDataFromFile(DATA_FILE + ".data");
		List<FloatArrayData> queryData = TestUtil.readFloatDataFromFile(DATA_FILE + ".query");
		// queryData = null;

//		t = new TestInsertionCost();
//		t.init(indexType, pageSize, bufferSize, data.get(0), t.readPivotList());
//		t.testInsertion(data, pivotCountConstant(24), sampleSize, queryData, queryRange, queryCount);
//		t.writeLog("exp-insertion-24.txt");
//		t.close();
		
		t = new TestInsertionCost();
		t.init(indexType, pageSize, bufferSize, data.get(0), t.readPivotList());
		t.testInsertion(data, pivotCountConstant(12), sampleSize, queryData, queryRange, queryCount);
		t.writeLog("exp-insertion-12.txt");
		t.close();

                t = new TestInsertionCost();
		t.init(indexType, pageSize, bufferSize, data.get(0), t.readPivotList());
		t.testInsertion(data, pivotCountLog(1, 1, 2), sampleSize, queryData, queryRange, queryCount);
		t.writeLog("exp-insertion-log2.txt");
		t.close();


                t = new TestInsertionCost();
		t.init(indexType, pageSize, bufferSize, data.get(0), t.readPivotList());
		t.testInsertion(data, pivotCountLog(1, 1, 10), sampleSize, queryData, queryRange, queryCount);
		t.writeLog("exp-insertion-log10.txt");
		t.close();


         }
}
