package test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import neustore.base.FloatArrayData;
import neustore.base.IntData;
import neustore.base.IntKey;
import neustore.base.LRUBuffer;
import ntnustore.btree.BTree;
import ntnustore.distance.Distance;
import ntnustore.distance.L2Distance;
import ntnustore.dlaesa.Cursor;
import ntnustore.dlaesa.DistanceCursor;
import ntnustore.dlaesa.KnnCursor;
import ntnustore.dlaesa.LaesaBTree;
import ntnustore.dlaesa.LaesaHeapFile;
import ntnustore.dlaesa.LaesaIndex;
import ntnustore.dlaesa.LaesaIndexList;
import ntnustore.dlaesa.LaesaRTree;
import ntnustore.dlaesa.Pair;

/**
 * TODO: Description of type.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class TestPivotCount {
    private static final String DATA_FILE = "data/nasa";
    //private static final String DATA_FILE = "data/unicoords";

    //private static final int[] PIVOT_COUNTS = new int[] { 1, 2, 4, 8, 16, 32, 64, 96, 128, 160, 192, 256, 384, 512, 768, 1024 };
    //private static final int[] PIVOT_COUNTS = new int[] { 1, 2, 4, 8, 16, 32, 64, 96, 128, 160, 192, 224, 256, 320, 384, 448, 512 };
    //private static final int[] PIVOT_COUNTS = new int[] { 1, 4, 16, 32 };
    private static final int[] PIVOT_COUNTS = new int[] {2};
    //private static final int[] PIVOT_COUNTS = new int[] { 448, 512 };

    //INDEX_TYPE: -1=HeapFile, 0=BTree, 1+=RTree (number indicates maximum dimensionality of R-tree)  
    private int INDEX_TYPE = 0;

    private static final int PAGE_SIZE = 8192;
    
    private static final int BUFFER_SIZE = 10000;
    //private static final float[] QUERY_RANGE = new float[]{0.6F, 0.8F, 1.0F, 1.2F, 1.4F};
    //private static final float[] QUERY_RANGE = new float[]{0.05F, 0.1F, 0.2F, 0.4F, 0.8F};
    private static final float[] QUERY_RANGE = new float[]{0.2F};

    //QUERY_COUNT: -1 = return candidate set only, 0 = perform exact distance check, X>1 = return X nearest neighbors
    private static final int QUERY_COUNT = 0;

    private static final boolean QUERY_FLUSH = true;
    
    private static final int QUERY_THREADS = 1;

    private List<FloatArrayData> queryStrings;
    
    private List<FloatArrayData> queryPivots;

    private LRUBuffer buffer;
    private BTree<IntKey, FloatArrayData> dataFile;

    /**
     * Create buffer and read data into dataFile.
     */
    private void readData() throws IOException {
        // read data from file
        List<FloatArrayData> fileData = TestUtil.readFloatDataFromFile(DATA_FILE + ".data");

        // create buffer and file
        buffer = new LRUBuffer(BUFFER_SIZE, PAGE_SIZE);
        dataFile = new BTree<IntKey, FloatArrayData>(buffer, "data/test", true, new IntKey(0), fileData.get(0));

        // insert data
        int key = 0;
        for (FloatArrayData data : fileData) {
            key++;
            dataFile.insert(new IntKey(key), data);
        }
        
        // read query and pivots
        queryStrings = TestUtil.readFloatDataFromFile(DATA_FILE + ".query");
        queryPivots = TestUtil.readFloatDataFromFile(DATA_FILE + ".pivot");
    }

    /**
     * Prints pivots generated by LaesaIndexList.getPivotCandiadate()
     */
    private void printPivots(int pivotCount) throws IOException {
        LaesaIndexList<FloatArrayData> indexList = new LaesaIndexList<FloatArrayData>();
        
        List<IntKey> pivots = indexList.getPivotCandidate(dataFile, new L2Distance(dataFile.search(new IntKey(1))), 1024);
        for (IntKey key : pivots) {
            FloatArrayData pivot = dataFile.search(key);
            String pivotStr = Arrays.toString(pivot.value());
            //System.out.println(pivotStr.substring(1, pivotStr.length() - 1).replace(", ", " "));
        }
    }

    /**
     * Creates an index based in the current settings and the number of pivots.
     */
    private LaesaIndexList<FloatArrayData> createIndex(int pivotCount) throws IOException, InterruptedException {
        // indexing
        buffer.flush(null);
        buffer.clearIOs();
        long t1 = System.nanoTime();

        LaesaIndexList<FloatArrayData> indexes = new LaesaIndexList<FloatArrayData>();

        int currentPivot = 0;

        ExecutorService executor = Executors.newFixedThreadPool((QUERY_THREADS <= 0 ? 1 : QUERY_THREADS));

        while (currentPivot < pivotCount) {
            final LaesaIndex<FloatArrayData> index;

            if (INDEX_TYPE > 0) {
                List<Distance<FloatArrayData>> distances = new ArrayList<Distance<FloatArrayData>>();

                int count = Math.min(INDEX_TYPE, pivotCount - currentPivot);
                while (distances.size() < count) {
                    FloatArrayData pivot = queryPivots.get(currentPivot);
                    distances.add(new L2Distance(pivot));
                    currentPivot++;
                }

                index = new LaesaRTree<FloatArrayData>(buffer, "data/test_pivot" + currentPivot, distances);
            } else {
                FloatArrayData pivot = queryPivots.get(currentPivot);
                Distance<FloatArrayData> distance = new L2Distance(pivot);
                currentPivot++;

                if (INDEX_TYPE == 0) {
                    index = new LaesaBTree<FloatArrayData>(buffer, "data/test_pivot" + currentPivot, distance);
                } else {
                    index = new LaesaHeapFile<FloatArrayData>(buffer, "data/test_pivot" + currentPivot, distance);
                }
            }

            // insert data
            executor.execute(new Runnable() {
                public void run() {
                    Cursor<IntKey, FloatArrayData> cursor = dataFile.query(new IntKey(0), new IntKey(Integer.MAX_VALUE));
                    while (cursor.next()) {
                        //index.insert(cursor.getKey(), cursor.getData());
                    }
                }
            });

            indexes.add(index);
        }

        executor.shutdown();
        executor.awaitTermination(10, TimeUnit.DAYS);
       
        buffer.flush(null);
        long t2 = System.nanoTime();

        addStats("index_" + pivotCount, t2 - t1);

        return indexes;
    }

    /**
     * queries the given index list with the given query and settings.
     */
    private void query(LaesaIndexList<FloatArrayData> index, FloatArrayData query, float range) throws Exception {
        if (QUERY_FLUSH) {
            buffer.flush(null);
        }
        buffer.clearIOs();
        long t1 = System.nanoTime();

        // perform query
        Cursor<IntData, Float> cursor;
        
        if (QUERY_THREADS > 0) {
            ExecutorService executor = Executors.newFixedThreadPool(QUERY_THREADS);
            cursor = index.query(query, range, executor, QUERY_THREADS);
            executor.shutdown();
        } else {
            cursor = index.query(query, range);
        }

        // filter inexact matches with KNN
        Distance<FloatArrayData> distance = new L2Distance(query);
        if (QUERY_COUNT > 0) {
            cursor = new KnnCursor<IntData, FloatArrayData>(cursor, dataFile, distance, range, QUERY_COUNT);
        } else if (QUERY_COUNT == 0) {
            cursor = new DistanceCursor<IntData, FloatArrayData>(cursor, dataFile, distance, range);
        }

        // count results
        int count = 0;
        while (cursor.next()) {
            count++;
        }

        long t2 = System.nanoTime();

        addStats("query_" + range, t2 - t1, count, distance.getCalculations());
    }

    private List<Pair<String, long[]>> stats = new ArrayList<Pair<String,long[]>>();

    private void addStats(String label, long time, long... extra) {
        long s[] = new long[5 + extra.length];

        s[0] = (int) (time / 1000 / 1000);
        int[] ios = buffer.getIOs();
        for (int i = 0; i < ios.length; i++) {
            s[i + 1] = ios[i];
        }

        for (int i = 0; i < extra.length; i++) {
            s[i + 5] = extra[i];
        }
        stats.add(new Pair<String, long[]>(label, s));
    }

    private void printStats() {
        Map<String, long[]> newStats = new LinkedHashMap<String, long[]>();
        Map<String, Integer> newStatsCount = new HashMap<String, Integer>();

        for (Pair<String, long[]> pair : stats) {
            long[] values = pair.value;
            int count = 1;
            if (newStats.containsKey(pair.key)) {
                long[] values2 = newStats.get(pair.key);
                for (int i = 0; i < values.length; i++) {
                    values[i] += values2[i];
                }
                count += newStatsCount.get(pair.key);
            }

            newStats.put(pair.key, values);
            newStatsCount.put(pair.key, count);
        }

        for (String key : newStats.keySet()) {
            //System.out.print(key);

            int count = newStatsCount.get(key);
            long[] values = newStats.get(key);
            for (long i : values) {
                //System.out.print("\t" + Math.round(i * 1000.0 / count) / 1000.0);
            }

            //System.out.print("\t");
        }
        //System.out.println();
        stats.clear();
    }

    public static void main(String[] args) throws Exception {
        TestPivotCount t = new TestPivotCount();
        if (args.length > 0) {
            t.INDEX_TYPE = Integer.parseInt(args[0]);
        }
        
        // create data file
        TestUtil.deleteFiles();
        t.readData();

        // if no pivots, print pivot candidates
        
        if (t.queryPivots == null) {
            t.printPivots(1024);
            return;
        }
        
        for (int pivotCount : PIVOT_COUNTS) {
            // create index
            LaesaIndexList<FloatArrayData> index;
            index = t.createIndex(pivotCount);

            // query
            for (float range : QUERY_RANGE) {
                for (FloatArrayData query : t.queryStrings) {
                    t.query(index, query, range);
                }
            }
            t.printStats();

            // close index
            index.close();
           // TestUtil.deleteFiles();
        }

        // close data file
        t.dataFile.close();
        //TestUtil.deleteFiles();*/
    }
}
