package test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import neustore.base.FloatArrayData;
import neustore.base.IntData;
import neustore.base.IntKey;
import ntnustore.distance.Distance;
import ntnustore.distance.L2Distance;
import ntnustore.distance.QFDistance;
import ntnustore.dlaesa.Cursor;
import ntnustore.dlaesa.DistanceCursor;
import ntnustore.dlaesa.KnnCursor;

/**
 * TODO: Description of type.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 * @author Svein Erik Bratsberg <sveinbra@idi.ntnu.no>
 */
public class TestPivotSelection extends AbstractTest<FloatArrayData> {
	private static final String DATA_FILE = "data/nasa";

        //private static final String DATA_FILE = "data/unicoords";
	private void testPivotSelection (List<FloatArrayData> fileData, PivotCount pivotCount, int sampleSize, List<FloatArrayData> queryData, float queryRange, int queryCount, float [][] matrix)
			throws Exception {

		int sumInsertCost = 0;
		long sumInsertTime = 0;

		int objectIndex = 0;
		while (objectIndex < fileData.size()) {
			// insert
			long startTime = System.nanoTime();
			int startCost = getDistanceCost();
//			int diskWriteIO = buffer.getIOs()[1];
			
			sampleSize = Math.min(sampleSize, fileData.size() - objectIndex);
			while (pivotCount.getPivotCount(objectIndex + sampleSize) > getPivotCount()) {
                                print("adding pivot", "\t");
                                print(objectIndex, "\n");
				addPivot();
			}
			for (int i = 0; i < sampleSize; i++) {
				insert(fileData.get(objectIndex + i));
			}
			objectIndex += sampleSize;
			buffer.force(null);

			print(objectIndex, "\t");
			
			int cost = getDistanceCost() - startCost;
			sumInsertCost += cost;
			print(cost, "\t");
			print(sumInsertCost, "\t");

			long time = (System.nanoTime() - startTime) / 1000000;
			sumInsertTime += time;
			print(time, "\t");
			print(sumInsertTime, "\t");
                        print("\n");
			
                }
			
			// query
	        if ((queryData != null)) {

                    indexList.generateStatistics();
                    int qi = 0;
                    long sumBtime = 0;
                    long sumTime = 0;
                    long sumCtime = 0;
                    long startTime = 0;
                    long endTime = 0;

                    startTime = System.nanoTime();
                    for (FloatArrayData query : queryData) {
                        //Distance<FloatArrayData> distance = new QFDistance(query, matrix);
                        Distance<FloatArrayData> distance = new L2Distance(query);
                        int ccount = 0;
                        Cursor<IntKey, FloatArrayData> dcursor = dataFile.query(new IntKey(0), new IntKey(objectIndex));
                        while (dcursor.next()) {
                            float currentDistance = distance.distance(dcursor.getData());
                            if (currentDistance <= queryRange) {
                                ccount++;
                            }
                        }
                    }
                    endTime = System.nanoTime();
                    sumCtime = (endTime-startTime)/1000000;

                    startTime = System.nanoTime();
                    for (FloatArrayData query : queryData) {
                        //Distance<FloatArrayData> distance = new QFDistance(query, matrix);
                        Distance<FloatArrayData> distance = new L2Distance(query);
                        Cursor<IntData, Float> cursor = indexList.query(query, queryRange);
                        cursor = new DistanceCursor<IntData, FloatArrayData>(cursor, dataFile, distance, queryRange);
                        int count = 0;
                        while (cursor.next()) {
                            count++;
                        }

                    }
                    endTime = System.nanoTime();
                    sumTime = (endTime-startTime)/1000000;

                    startTime = System.nanoTime();
                    for (FloatArrayData query : queryData) {
                        //Distance<FloatArrayData> distance = new QFDistance(query, matrix);
                        Distance<FloatArrayData> distance = new L2Distance(query);
                        Cursor<IntData, Float> cursor = indexList.queryKBest(query, queryRange,2);
                        cursor = new DistanceCursor<IntData, FloatArrayData>(cursor, dataFile, distance, queryRange);
                        int count = 0;
                        while (cursor.next()) {
                            count++;
                        }

                    }
                    endTime = System.nanoTime();
                    sumBtime = (endTime-startTime)/1000000;

                    System.out.println("sumBtime=" + sumBtime + " sumTime=" + sumTime + " sumCtime=" + sumCtime);
		}
	}

	private List<Distance<FloatArrayData>> readPivotList(float [][] matrix) throws IOException {
		List<Distance<FloatArrayData>> result = new ArrayList<Distance<FloatArrayData>>();

		List<FloatArrayData> pivots = TestUtil.readFloatDataFromFile(DATA_FILE + ".pivot");
		for (FloatArrayData pivot : pivots) {
			Distance<FloatArrayData> distance = new L2Distance();
			distance.initialize(pivot);
                        //Distance<FloatArrayData> distance = new QFDistance(pivot,matrix);
			result.add(distance);
		}

		return result;
	}
	
	public static void main(String[] args) throws Exception {
		int indexType = 4;
		int pageSize = 8192;
		int bufferSize = 10000; //TODO
		int sampleSize = 2000;
		int queryCount = 0;
		float queryRange = 0.2F;
		
                TestUtil.deleteFiles();
		TestPivotSelection t;
		List<FloatArrayData> data = TestUtil.readFloatDataFromFile(DATA_FILE + ".data");
		List<FloatArrayData> queryData = TestUtil.readFloatDataFromFile(DATA_FILE + ".query");
		// queryData = null;

                // create matrix in case of QFD distance
                int sz = 0;
                for (FloatArrayData query : queryData) {
                    sz = query.size();
                    break;
                }
                float[][] matrix = new float[sz][sz];
                // create identity matrix
                for (int i = 0; i < sz; i++) {
                    for (int j = 0; j < sz; j++) {
                        if (i == j)
                            matrix[i][j] = 1.0F;
                        else
                            matrix[i][j] = 0;
                    }
                }

//		t = new TestInsertionCost();
//		t.init(indexType, pageSize, bufferSize, data.get(0), t.readPivotList());
//		t.testInsertion(data, pivotCountConstant(24), sampleSize, queryData, queryRange, queryCount);
//		t.writeLog("exp-insertion-24.txt");
//		t.close();
		
		t = new TestPivotSelection();
		t.init(indexType, pageSize, bufferSize, data.get(0), t.readPivotList(matrix));
		t.testPivotSelection(data, pivotCountConstant(12), sampleSize, queryData, queryRange, queryCount, matrix);
		t.writeLog("exp-insertion-12.txt");
		t.close();

/*                t = new TestInsertionCost();
		t.init(indexType, pageSize, bufferSize, data.get(0), t.readPivotList());
		t.testInsertion(data, pivotCountLog(1, 1, 2), sampleSize, queryData, queryRange, queryCount);
		t.writeLog("exp-insertion-log2.txt");
		t.close();


                t = new TestInsertionCost();
		t.init(indexType, pageSize, bufferSize, data.get(0), t.readPivotList());
		t.testInsertion(data, pivotCountLog(1, 1, 10), sampleSize, queryData, queryRange, queryCount);
		t.writeLog("exp-insertion-log10.txt");
		t.close();
*/

         }
}
