package test;

import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.Random;

import uk.ac.shef.wit.simmetrics.similaritymetrics.*;

import neustore.base.Data;
import neustore.base.FloatArrayData;
import neustore.base.StringData;
import ntnustore.distance.ChebyshevDistance;
import ntnustore.distance.Distance;
import ntnustore.distance.L2Distance;
import ntnustore.distance.LXDistance;
import ntnustore.distance.LevenshteinDistance;
import ntnustore.distance.SimMetricDistance;
import ntnustore.distance.LevenshteinWordDistance;

/**
 * TODO: Description of type.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class TestRange<T extends Data<?>> {
	private static final String DATA_FILE = "data/nasa";

	private List<T> dataList;

	private void printDistanceDistribution(Distance<T> distance, double max,
			double interval, int knn) {
		long[] rangeCount = new long[(int) Math.ceil(max / interval)];
		long[] rangeKnn = new long[rangeCount.length];
		int totalCount = 50;// dataList.size();

		List<T> objects = new ArrayList<T>(dataList);
		Collections.shuffle(objects);

		for (int p = 0; p < totalCount; p++) {
			T object = objects.get(p);

			double dists[] = new double[dataList.size()];
			distance.initialize(object);
			for (int i = 0; i < dataList.size(); i++) {
				dists[i] = distance.distance(dataList.get(i));
			}

			long count[] = new long[rangeCount.length];
			for (int i = 0; i < dists.length; i++) {
				int inter = Math.max(0, (int) Math
						.floor((dists[i] - 0.0000000001) / interval));
				if (inter < count.length) {
					count[inter]++;
				}
			}

			int nn = 0;
			for (int i = 0; i < rangeCount.length; i++) {
				rangeCount[i] += count[i];

				nn += count[i];
				if (nn >= knn) {
					rangeKnn[i]++;
				}
			}
		}

		System.out.print("Interval");
		System.out.print("\tN(R)");
		System.out.print("\tFrequency");
		System.out.print("\tDistribution");
		System.out.print("\tK>=" + knn + " %");
		System.out.println();

		long rangeSum = 0;
		for (int i = 0; i < rangeCount.length; i++) {
			System.out.print((int) (interval * (i + 1) * 10000) / 10000.0
					+ "\t");

			rangeSum += rangeCount[i];
			System.out.print(rangeSum * 1.0 / totalCount + "\t");

			System.out.print(rangeCount[i] * 1.0 / totalCount + "\t");

			System.out.print(rangeCount[i] * 1.0 / totalCount / dataList.size()
					+ "\t");

			System.out.print(rangeKnn[i] * 1.0 / totalCount + "\t");

			System.out.println();
		}
	}

	private double[] sampleDistances(Distance<T> distance, int count) {
		Random r1 = new Random(), r2 = new Random();
		double dists[] = new double[count];

		for (int i = 0; i < count; i++) {
			T o1 = dataList.get(r1.nextInt(dataList.size()));
			T o2 = dataList.get(r2.nextInt(dataList.size()));

			distance.initialize(o1);
			dists[i] = distance.distance(o2);
		}

		return dists;
	}

	private void printDistanceDistribution(double[] dists, int intervals) {
		double mean = mean(dists);
		//mean = normalize(dists, mean);
		
		double variance = 0.0;
		for (double d : dists) {
			variance += (d - mean)*(d - mean);
		}
		variance = (variance) / (dists.length - 1);
		
		double dimensionality = mean * mean / (2 * variance);
		
		double interval = mean * 2 / intervals;
		int count = 0;
		int histogram[] = new int[intervals];
		
		for (double d : dists) {
			int i = (int)Math.round(d / interval);
			if ((i >= 0) && (i < histogram.length)) { 
				histogram[i]++;
				count++;
			}
		}

		// Print results
		NumberFormat format = NumberFormat.getInstance(Locale.US);
		format.setGroupingUsed(false);
		format.setMaximumFractionDigits(20);

		System.out.println("Size:\t" + dists.length);
		System.out.println("Mean:\t" + format.format(mean));
		System.out.println("Variance:\t" + format.format(variance));
		System.out.println("Dimensionality:\t" + format.format(dimensionality));
		
		for (int i = 0; i < histogram.length; i++) {
			System.out.println(format.format(i * interval) + "\t" + format.format(histogram[i] / interval / count));
		}
	}

	private double mean(double[] dists) {
		double sum = 0.0;
		for (double d : dists) {
			sum += d;
		}
		
		return sum / dists.length;
	}

	private double normalize(double[] dists, double mean) {
		for (int i = 0; i < dists.length; i++) {
			dists[i] = dists[i] / (mean * 2);
		}
		
		return mean / (mean * 2); // 0.5
	}

	/**
	 * Fetch and print count random objects from the data set.
	 * 
	 * @param count
	 *            number of objects to print
	 */
	private void printRandomObjects(int count) {
		List<T> objects = new ArrayList<T>(dataList);
		Collections.shuffle(objects);

		for (int i = 0; i < count; i++) {
			T object = objects.get(i);
			System.out.println(object.value());
		}
	}

	public static void main(String[] args) throws Exception {
		//TestUtil.extractTrecDataFromFiles("C:\\ntnu\\master\\Trec Utdrag\\GX000", 2048, "R:/trecGX000.txt");
		//TestUtil.extractTrecDataFromFiles("R:/GX002", 2048, "R:/trecGX002.txt");
		//TestUtil.extractTrecDataFromFiles("data/trec", 2048, "R:/trec3.txt");
		
//		TestRange<FloatArrayData> t = new TestRange<FloatArrayData>();
//		t.dataList = TestUtil.readFloatDataFromFile("data/nasa.data");
//		double dists[] = t.sampleDistances(new LXDistance(1F), 100000);
//		t.printDistanceDistribution(dists, 100);
//
		TestRange<FloatArrayData> t = new TestRange<FloatArrayData>();
              t = new TestRange<FloatArrayData>();
		t.dataList = TestUtil.readFloatDataFromFile("data/nasa.data");
		double dists[] = t.sampleDistances(new L2Distance(), 100000);
		t.printDistanceDistribution(dists, 100);

//                TestRange<FloatArrayData> t = new TestRange<FloatArrayData>();
//                t = new TestRange<FloatArrayData>();
//		t.dataList = TestUtil.readFloatDataFromFile("data/unicoords.data");
//		double dists[] = t.sampleDistances(new L2Distance(), 1000000);
//		t.printDistanceDistribution(dists, 100);
//
//		t = new TestRange<FloatArrayData>();
//		t.dataList = TestUtil.readFloatDataFromFile("data/nasa.data");
//		dists = t.sampleDistances(new LXDistance(5F), 100000);
//		t.printDistanceDistribution(dists, 100);
//		
//		t = new TestRange<FloatArrayData>();
//		t.dataList = TestUtil.readFloatDataFromFile("data/nasa.data");
//		dists = t.sampleDistances(new ChebyshevDistance(), 100000);
//		t.printDistanceDistribution(dists, 100);

		
//		TestRange<FloatArrayData> t = new TestRange<FloatArrayData>();
//		t.dataList = TestUtil.readFloatDataFromFile("data/colors.data");
//		double dists[] = t.sampleDistances(new LXDistance(1F), 100000);
//		t.printDistanceDistribution(dists, 100);
//
//		t = new TestRange<FloatArrayData>();
//		t.dataList = TestUtil.readFloatDataFromFile("data/colors.data");
//		dists = t.sampleDistances(new L2Distance(), 100000);
//		t.printDistanceDistribution(dists, 100);
//
//		t = new TestRange<FloatArrayData>();
//		t.dataList = TestUtil.readFloatDataFromFile("data/colors.data");
//		dists = t.sampleDistances(new LXDistance(5F), 100000);
//		t.printDistanceDistribution(dists, 100);
//		
//		t = new TestRange<FloatArrayData>();
//		t.dataList = TestUtil.readFloatDataFromFile("data/colors.data");
//		dists = t.sampleDistances(new ChebyshevDistance(), 100000);
//		t.printDistanceDistribution(dists, 100);

//		TestRange<StringData> t = new TestRange<StringData>();
//		t.dataList = TestUtil.readStringDataFromFile("data/GX000_002.txt", 2048);
//		double dists[] = t.sampleDistances(new LevenshteinWordDistance(), 100000);
//		t.printDistanceDistribution(dists, 100);
//
//		TestRange<StringData> t = new TestRange<StringData>();
//		t.dataList = TestUtil.readStringDataFromFile("data/GX000_002.txt", 2048);
//		double[] dists = t.sampleDistances(new LevenshteinDistance(), 100000);
//		t.printDistanceDistribution(dists, 100);

//		TestRange<StringData> t = new TestRange<StringData>();
//		t.dataList = TestUtil.readTrecDataFromFiles("data/trec/", 2048);
//		long t1 = System.nanoTime();
//		InterfaceStringMetric sim = new QGramsDistance(); 
//		double dists[] = t.sampleDistances(new SimMetricDistance(sim), 100);
//		long t2 = System.nanoTime();
//		System.out.println(sim.getClass().getSimpleName() + "\tTime: " + (t2 - t1) / 1000000 + " ms");
//		t.printDistanceDistribution(dists, 50);
	}
}
