package test;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import neustore.base.FloatArrayData;
import neustore.base.StringData;
import ntnustore.distance.Distance;

/**
 * Utility methods for testing.
 * 
 * @version $Id:$
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class TestUtil {
	public static List<FloatArrayData> readFloatDataFromFile(String fileName)
			throws IOException {
		List<FloatArrayData> result = new ArrayList<FloatArrayData>();

		List<String> lines = readLinesFromFile(fileName);

		for (String line : lines) {
			String[] s = line.split(" ");
			float[] f = new float[s.length];
			for (byte i = 0; i < f.length; i++) {
				f[i] = Float.parseFloat(s[i]);
			}

			FloatArrayData data = new FloatArrayData(f);
			result.add(data);

			if (result.get(0).size() != data.size()) {
				throw new IOException("Invalid size: " + s);
			}
		}

		return result;
	}

	public static List<StringData> readStringDataFromFile(String fileName, int maxLen)
			throws IOException {
		List<StringData> result = new ArrayList<StringData>();

		List<String> lines = readLinesFromFile(fileName);

		for (String line : lines) {
			result.add(new StringData(maxLen, line));
		}

		return result;
	}

	public static void extractTrecDataFromFiles(String dirName, int maxLen, String outFile) throws IOException {
		BufferedWriter writer = new BufferedWriter(new FileWriter(outFile, false));

		File[] files = new File(dirName).listFiles();
		for (File file : files) {
			List<StringData> content = readTrecDataFromFile(file.getPath(), maxLen);
			System.out.println(file.getName() + "\t" + content.size());
			
			for (StringData stringData : content) {
				writer.write(stringData.value());
				writer.write("\r\n");
			}
		}

		writer.close();
	}

	public static List<StringData> readTrecDataFromFiles(String dirName,
			int maxLen) throws IOException {
		List<StringData> result = new ArrayList<StringData>();

		File[] files = new File(dirName).listFiles();
		for (File file : files) {
			result.addAll(readTrecDataFromFile(file.getPath(), maxLen));
		}

		return result;
	}

	public static List<StringData> readTrecDataFromFile(String fileName,
			int maxLen) throws IOException {
		List<StringData> result = new ArrayList<StringData>();

		List<String> lines = readLinesFromFile(fileName);
		StringBuffer sb = new StringBuffer();

		for (String line : lines) {
			if ("</DOCHDR>".equals(line)) {
				sb = new StringBuffer();
			} else if ("</DOC>".equals(line)) {
				String content = sb.toString();
				content = content.replaceAll("\n", " ");
				content = content.toLowerCase();
				content = content.replaceAll("<script.*?/script>", " ");
				content = content.replaceAll("<!--.*?-->", " ");
				content = content.replaceAll("<.*?>", " ");
				content = content.replaceAll("&.{1,6};", " ");
				content = content.replaceAll("[^a-z0-9]+", " ");
				content = content.trim();
				if (content.length() > maxLen) {
					content = content.substring(0, maxLen);
				} else if (content.length() == 0) {
					continue;
				}

				result.add(new StringData(maxLen, content));
			} else {
				sb.append(line);
				sb.append("\n");
			}
		}

		return result;
	}

	public static List<String> readLinesFromFile(String fileName)
			throws IOException {
		BufferedReader br = new BufferedReader(new FileReader(fileName));
		List<String> result = new ArrayList<String>();

		String line;
		while ((line = br.readLine()) != null) {
			result.add(line);
		}

		return result;
	}

	/**
	 * Deletes all files used in the test
	 */
	public static void deleteFiles() {
		new File("data/test").delete();
		new File("data/pivots.hashmap").delete();
		new File("data/pivot.keys").delete();
		for (int i = 0; i < 45000; i++) {
			new File("data/pivotIntKey[value=" + i + "].HeapFile").delete();
			new File("data/pivotIntKey[value=" + i + "].BTree").delete();
		}
	}

	public static <T> List<Distance<T>> createDistances(List<T> objects,
			Class<? extends Distance<T>> clazz) throws InstantiationException,
			IllegalAccessException {
		List<Distance<T>> result = new ArrayList<Distance<T>>();

		for (T object : objects) {
			Distance<T> distance = clazz.newInstance();
			distance.initialize(object);
			result.add(distance);
		}

		return result;
	}
	
	public static void _main(String[] args) {
		String content = "secretary of state disclaimer <!-- function mm_swapimgrestore() { //v3.0 var i,x,a=document.mm_sr; for(i=0;a&&i<a.length&&(x=a[i])&&x.osrc;i++) x.src=x.osrc; } function mm_preloadimages() { //v3.0 var d=document; if(d.images){ if(!d.mm_p) d.mm_p=new array(); var i,j=d.mm_p.length,a=mm_preloadimages.arguments; for(i=0; i<a.length; i++) if (a[i].indexof(\"#\")!=0){ d.mm_p[j]=new image; d.mm_p[j++].src=a[i];}} } function mm_findobj(n, d) { //v4.0 var p,i,x; if(!d) d=document; if((p=n.indexof(\"?\"))>0&&parent.frames.length) { d=parent.frames[n.substring(p+1)].document; n=n.substring(0,p);} if(!(x=d[n])&&d.all) x=d.all[n]; for (i=0;!x&&i<d.forms.length;i++) x=d.forms[i][n]; for(i=0;!x&&d.layers&&i<d.layers.length;i++) x=mm_findobj(n,d.layers[i].document); if(!x && document.getelementbyid) x=document.getelementbyid(n); return x; } function mm_swapimage() { //v3.0 var i,j=0,x,a=mm_swapimage.arguments; document.mm_sr=new array; for(i=0;i<(a.length-2);i+=3) if ((x=mm_findobj(a[i]))!=null){document.mm_sr[j++]=x; if(!x.osrc) x.osrc=x.src; x.src=a[i+2];} } //--> search &nbsp;&nbsp;|&nbsp;&nbsp; site index &nbsp;&nbsp;|&nbsp;&nbsp; help &nbsp;&nbsp;|&nbsp;&nbsp; contacts&nbsp;&nbsp; |&nbsp;&nbsp; press office <!-- function mm_jumpmenu(targ,selobj,restore){ //v3.0 eval(targ+\".location='\"+selobj.options[selobj.selectedindex].value+\"'\"); if (restore) selobj.selectedindex=0; } //--> quick links: select a topic about - secretary of state bio - assistant secretary bio - press releases - history of the office - constitutional duties - employment opportunities - procurement opportunities elections - voter information - conducting your elections - candidates - project v.o.t.e. - election returns - election law opinions texas register texas administrative code open meetings statutory documents - notary public - state seal corporations - filing options &amp; aids - public information - copies &amp; certificates - tax information uniform commercial code - fees - electronic filing (edi) border &amp; mexican affairs - colonias international p";
		System.out.println(content);
		content = content.toLowerCase();
		content = content.replaceAll("<script.*?/script>", " ");
		content = content.replaceAll("<!--.*?-->", " ");
		content = content.replaceAll("<.*?>", " ");
		content = content.replaceAll("&.{1,6};", " ");
		content = content.replaceAll("[^a-z0-9]+", " ");
		content = content.trim();
		System.out.println(content);
	}
}
