package ntnustore.distance;

import static org.junit.Assert.*;

import neustore.base.StringData;

import org.junit.Before;
import org.junit.Test;

/**
 * JUnit test case for class {@link ntnustore.distance.LevenshteinDistanceTest}.
 * 
 * @version $Id: LevenshteinDistanceTest.java 60 2009-03-15 22:43:49Z erikbagg $
 * @author Erik Bagge Ottesen <erikbagg@idi.ntnu.no>
 */
public class LevenshteinDistanceTest {
    public static final String TEST_WORD = "abcdef";
    public static final String TEST_STRINGS[] = new String[] { "", "fedcba", "acf", "af", "aof", "a", "x", "aklmnf",
            "ojankilecmneuf", "ojmaceduf", "xxxxxxxxx", "abcdefxxxxxxxxx" };
    public static final double TEST_DISTANCE[] = new double[] { 6.0, 6.0, 3.0, 4.0, 4.0, 5.0, 6.0, 4.0, 10.0, 6.0, 9.0,
            9.0 };

    LevenshteinDistance distance;

    /**
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception {
        distance = new LevenshteinDistance(new StringData(100, TEST_WORD));
    }

    /**
     * Test method for
     * {@link ntnustore.distance.LevenshteinDistance#distance(java.lang.String)}.
     */
    @Test
    public void testGetDistance() {
        for (int i = 0; i < TEST_STRINGS.length; i++) {
            assertEquals(TEST_DISTANCE[i], distance.distance(new StringData(100, TEST_STRINGS[i])), 0.0001);
        }
    }

    /**
     * Test method for
     * {@link ntnustore.distance.LevenshteinDistance#distance(java.lang.String)}.
     */
    @Test
    public void testGetDistanceReversed() {
        for (int i = 0; i < TEST_STRINGS.length; i++) {
            assertEquals(TEST_DISTANCE[i], new LevenshteinDistance(new StringData(100, TEST_STRINGS[i])).distance(new StringData(100, TEST_WORD)), 0.0001);
        }
    }

    /**
     * Test method for
     * {@link ntnustore.distance.LevenshteinDistance#distance(java.lang.String)}.
     */
    @Test
    public void testGetDistanceTriangle() {
        double d[][] = new double[TEST_STRINGS.length][TEST_STRINGS.length];
        for (int i = 0; i < TEST_STRINGS.length; i++) {
            LevenshteinDistance dist = new LevenshteinDistance(new StringData(100, TEST_STRINGS[i]));
            for (int j = 0; j < TEST_STRINGS.length; j++) {
                d[i][j] = dist.distance(new StringData(100, TEST_STRINGS[j]));
            }
        }
    }

    /**
     * Test method for
     * {@link ntnustore.distance.LevenshteinDistance#distance(java.lang.String)}.
     */
    @Test
    public void testGetDistanceDelete12() {
        for (int i = 0; i < TEST_WORD.length(); i++) {
            StringBuffer sb1 = new StringBuffer(TEST_WORD);
            sb1.deleteCharAt(i);
            assertEquals(1.0, distance.distance(new StringData(100, sb1.toString())), 0.0001);

            for (int j = i; j < sb1.length(); j++) {
                StringBuffer sb2 = new StringBuffer(sb1);
                sb2.deleteCharAt(j);
                assertEquals(2.0, distance.distance(new StringData(100, sb2.toString())), 0.0001);
            }
        }
    }

    /**
     * Test method for
     * {@link ntnustore.distance.LevenshteinDistance#distance(java.lang.String)}.
     */
    @Test
    public void testGetDistanceInsert12() {
        for (int i = 0; i < TEST_WORD.length(); i++) {
            StringBuffer sb1 = new StringBuffer(TEST_WORD);
            sb1.insert(i, 'X');
            assertEquals(1.0, distance.distance(new StringData(100, sb1.toString())), 0.0001);

            for (int j = i; j < sb1.length(); j++) {
                StringBuffer sb2 = new StringBuffer(sb1);
                sb2.insert(j, 'X');
                assertEquals(2.0, distance.distance(new StringData(100, sb2.toString())), 0.0001);
            }
        }
    }

    /**
     * Test method for
     * {@link ntnustore.distance.LevenshteinDistance#distance(java.lang.String)}.
     */
    @Test
    public void testGetDistanceReplace12() {
        for (int i = 0; i < TEST_WORD.length(); i++) {
            StringBuffer sb1 = new StringBuffer(TEST_WORD);
            sb1.replace(i, i + 1, "X");
            assertEquals(1.0, distance.distance(new StringData(100, sb1.toString())), 0.0001);

            for (int j = i + 1; j < sb1.length(); j++) {
                StringBuffer sb2 = new StringBuffer(sb1);
                sb2.replace(j, j + 1, "X");
                assertEquals(2.0, distance.distance(new StringData(100, sb2.toString())), 0.0001);
            }
        }
    }

    /**
     * Test method for
     * {@link ntnustore.distance.LevenshteinDistance#distance(java.lang.String)}.
     */
    @Test(expected = NullPointerException.class)
    public void testGetDistanceNull() {
        distance.distance(null);
    }

    /**
     * Test method for
     * {@link ntnustore.distance.LevenshteinDistance#getCalculations()}.
     */
    @Test
    public void testGetCalculations() {
        for (int i = 0; i < 10; i++) {
            assertEquals(i, distance.getCalculations());
            distance.distance(new StringData(100, ""));
        }
    }
}
